// Rony Sason - ronysason@gmail.com
// Guy Assif  - guyks1@gmail.com

/**
   This code is used to manage a co-working spaces meeting rooms.
   Full explenation: // Instructable link
*/

#include <ESP8266HTTPClient.h>
#include <ESP8266WiFi.h>

// Define the Wifi
#define WIFI_SSID "Assif1"
#define WIFI_PASSPHRASE "1122334455"

//Define Sound Paramaters
#define MicPin A0
#define TalkingVol 50
int inputMic;

//PIR Sensor
long sensorState;

// true = room is free of people
// false = room is busy
bool roomFree = false;

// Wait time between checks
long timer = 1000 * 60 * 10; // 10 minutes

void setup()
{
  Serial.begin(115200);
  WiFi.begin(WIFI_SSID, WIFI_PASSPHRASE);

  //Wait for the WiFI connection completion
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.println("Waiting for connection");
  }

  pinMode(MicPin, OUTPUT);
  pinMode(D7, INPUT);

  inputMic = 0;

  sensorState = LOW;
}


/**
  If there is a WIFI connection, and we detected that we want to free the room,
  we activate a WEBHOOK we created that sends (threw Zapier)
  the user who created the event an eMail to ask him if we can FREE the calender.
*/
void loop()
{
  if (roomCheck())
  {
    Serial.println("Room is BUSY");
  } else {
    Serial.println("Room is FREE");

    if (WiFi.status() == WL_CONNECTED) {      //Check WiFi connection status
      HTTPClient http;                        //Declare object of class HTTPClient
      http.begin("https://hooks.zapier.com/hooks/catch/3543285/gblh3n/?test=t");       //Specify request destination
      http.end();                             //Close connection

      Serial.println("Sent request to delete event");
    } else {
      Serial.println("Error in WiFi connection");
    }
  }

  // Wait before next check
  delay(timer);
}


/**
 * If someone is talking, return TRUE
 */
bool readSound()
{
  inputMic = analogRead(MicPin);
  delay(10);

  if (inputMic > TalkingVol) return true;
  else return false;
}

/**
   Reads PIR sensor value in some time interval
*/
bool readSensor()
{
  sensorState = digitalRead(D7);
  if (sensorState == HIGH) {
    Serial.println("Someone is in the room!");
    delay(1000);
    return true;
  }
  else {
    Serial.println("No motion detected");
    delay(1000);
    return false;
  }
}


/**
  Check the room (sound & motion) for 20 times (max).
  If it detects action in the room more then a few times, returns TRUE.
*/
bool roomCheck()
{
  int talkNoise = 0;
  int motionRate;         // Checks for motion

  for (int j = 0; j < 20; j++)
  {
    int noiseDetected = 0;
    for (int i = 0; i < 1000; i++)
    {
      if (readSound())
        noiseDetected++;
    }

    if (readSensor() == true)
      motionRate++;

    Serial.println(noiseDetected);

    if (noiseDetected > 10) talkNoise++;

    // If Motion or Noise is detected for more then a few times
    // We can stop checking and return true (room is BUSY)
    if (talkNoise > 6 || motionRate > 10) return true;
  }

  return false;
}
